@@ -50,6 +50,8 @@
 #define mode_single 2
 #define mode_multiple 3
 
+#define BUFFERSIZE 1024
+
 #define ssl_all 0
 #define ssl_v2 1
 #define ssl_v3 2
@@ -74,11 +76,11 @@ const char *program_banner = "                   _\n"
                              "          / __/ __| / __|/ __/ _` | '_ \\\n"
                              "          \\__ \\__ \\ \\__ \\ (_| (_| | | | |\n"
                              "          |___/___/_|___/\\___\\__,_|_| |_|\n\n"
-                             "                    Version 1.6\n"
+                             "                    Version 1.7.1\n"
                              "              http://www.titania.co.uk\n"
                              "     Copyright (C) 2007-2008 Ian Ventura-Whiting\n";
-const char *program_version = "sslscan version 1.6\nhttp://www.titania.co.uk\nCopyright (C) 2007-2008 Ian Ventura-Whiting\n";
-const char *xml_version = "1.6";
+const char *program_version = "sslscan version 1.7.1\nhttp://www.titania.co.uk\nCopyright (C) 2007-2008 Ian Ventura-Whiting\n";
+const char *xml_version = "1.7";
 
 
 struct sslCipher
@@ -98,8 +100,10 @@ struct sslCheckOptions
 	char host[512];
 	int port;
 	int noFailed;
+	int starttls;
 	int sslVersion;
 	int targets;
+	int pout;
 
 	// File Handles...
 	FILE *xmlOutput;
@@ -231,6 +235,7 @@ int tcpConnect(struct sslCheckOptions *options)
 {
 	// Variables...
 	int socketDescriptor;
+	unsigned char buffer[BUFFERSIZE];
 	struct sockaddr_in localAddress;
 	int status;
 
@@ -261,6 +266,37 @@ int tcpConnect(struct sslCheckOptions *options)
 		return 0;
 	}
 
+	// If STARTTLS is required...
+	if (options->starttls == true)
+	{
+		memset(buffer, 0, BUFFERSIZE);
+		recv(socketDescriptor, buffer, BUFFERSIZE - 1, 0);
+		if (strncmp(buffer, "220", 3) != 0)
+		{
+			close(socketDescriptor);
+			printf("%s    ERROR: The host %s on port %d did not appear to be an SMTP service.%s\n", COL_RED, options->host, options->port, RESET);
+			return 0;
+		}
+		send(socketDescriptor, "EHLO titania.co.uk\r\n", 20, 0);
+		memset(buffer, 0, BUFFERSIZE);
+		recv(socketDescriptor, buffer, BUFFERSIZE - 1, 0);
+		if (strncmp(buffer, "250", 3) != 0)
+		{
+			close(socketDescriptor);
+			printf("%s    ERROR: The SMTP service on %s port %d did not respond with status 250 to our HELO.%s\n", COL_RED, options->host, options->port, RESET);
+			return 0;
+		}
+		send(socketDescriptor, "STARTTLS\r\n", 10, 0);
+		memset(buffer, 0, BUFFERSIZE);
+		recv(socketDescriptor, buffer, BUFFERSIZE - 1, 0);
+		if (strncmp(buffer, "220", 3) != 0)
+		{
+			close(socketDescriptor);
+			printf("%s    ERROR: The SMTP service on %s port %d did not appear to support STARTTLS.%s\n", COL_RED, options->host, options->port, RESET);
+			return 0;
+		}
+	}
+
 	// Return
 	return socketDescriptor;
 }
@@ -432,37 +468,55 @@ int testCipher(struct sslCheckOptions *options, struct sslCipher *sslCipherPoint
 					{
 						if (options->xmlOutput != 0)
 							fprintf(options->xmlOutput, "accepted\" sslversion=\"");
-						printf("    Accepted  ");
+						if (options->pout == true)
+							printf("|| Accepted || ");
+						else
+							printf("    Accepted  ");
 					}
 					else if (cipherStatus == 0)
 					{
 						if (options->xmlOutput != 0)
 							fprintf(options->xmlOutput, "rejected\" sslversion=\"");
-						printf("    Rejected  ");
+						if (options->pout == true)
+							printf("|| Rejected || ");
+						else
+							printf("    Rejected  ");
 					}
 					else
 					{
 						if (options->xmlOutput != 0)
 							fprintf(options->xmlOutput, "failed\" sslversion=\"");
-						printf("    Failed    ");
+						if (options->pout == true)
+							printf("|| Failed || ");
+						else
+							printf("    Failed    ");
 					}
 					if (sslCipherPointer->sslMethod == SSLv2_client_method())
 					{
 						if (options->xmlOutput != 0)
 							fprintf(options->xmlOutput, "SSLv2\" bits=\"");
-						printf("SSLv2  ");
+						if (options->pout == true)
+							printf("SSLv2 || ");
+						else
+							printf("SSLv2  ");
 					}
 					else if (sslCipherPointer->sslMethod == SSLv3_client_method())
 					{
 						if (options->xmlOutput != 0)
 							fprintf(options->xmlOutput, "SSLv3\" bits=\"");
-						printf("SSLv3  ");
+						if (options->pout == true)
+							printf("SSLv3 || ");
+						else
+							printf("SSLv3  ");
 					}
 					else
 					{
 						if (options->xmlOutput != 0)
 							fprintf(options->xmlOutput, "TLSv1\" bits=\"");
-						printf("TLSv1  ");
+						if (options->pout == true)
+							printf("TLSv1 || ");
+						else
+							printf("TLSv1  ");
 					}
 					if (sslCipherPointer->bits < 10)
 						tempInt = 2;
@@ -470,7 +524,10 @@ int testCipher(struct sslCheckOptions *options, struct sslCipher *sslCipherPoint
 						tempInt = 1;
 					else
 						tempInt = 0;
-					printf("%d bits  ", sslCipherPointer->bits);
+					if (options->pout == true)
+						printf("%d || ", sslCipherPointer->bits);
+					else
+						printf("%d bits  ", sslCipherPointer->bits);
 					while (tempInt != 0)
 					{
 						tempInt--;
@@ -478,7 +535,10 @@ int testCipher(struct sslCheckOptions *options, struct sslCipher *sslCipherPoint
 					}
 					if (options->xmlOutput != 0)
 						fprintf(options->xmlOutput, "%d\" cipher=\"%s\" />\n", sslCipherPointer->bits, sslCipherPointer->name);
-					printf("%s\n", sslCipherPointer->name);
+					if (options->pout == true)
+						printf("%s ||\n", sslCipherPointer->name);
+					else
+						printf("%s\n", sslCipherPointer->name);
 				}
 
 				// Free SSL object
@@ -556,19 +616,28 @@ int defaultCipher(struct sslCheckOptions *options, SSL_METHOD *sslMethod)
 							{
 								if (options->xmlOutput != 0)
 									fprintf(options->xmlOutput, "  <defaultcipher sslversion=\"SSLv2\" bits=\"");
-								printf("    SSLv2  ");
+								if (options->pout == true)
+									printf("|| SSLv2 || ");
+								else
+									printf("    SSLv2  ");
 							}
 							else if (sslMethod == SSLv3_client_method())
 							{
 								if (options->xmlOutput != 0)
 									fprintf(options->xmlOutput, "  <defaultcipher sslversion=\"SSLv3\" bits=\"");
-								printf("    SSLv3  ");
+								if (options->pout == true)
+									printf("|| SSLv3 || ");
+								else
+									printf("    SSLv3  ");
 							}
 							else
 							{
 								if (options->xmlOutput != 0)
 									fprintf(options->xmlOutput, "  <defaultcipher sslversion=\"TLSv1\" bits=\"");
-								printf("    TLSv1  ");
+								if (options->pout == true)
+									printf("|| TLSv1 || ");
+								else
+									printf("    TLSv1  ");
 							}
 							if (SSL_get_cipher_bits(ssl, &tempInt2) < 10)
 								tempInt = 2;
@@ -576,7 +645,10 @@ int defaultCipher(struct sslCheckOptions *options, SSL_METHOD *sslMethod)
 								tempInt = 1;
 							else
 								tempInt = 0;
-							printf("%d bits  ", SSL_get_cipher_bits(ssl, &tempInt2));
+							if (options->pout == true)
+								printf("%d bits || ", SSL_get_cipher_bits(ssl, &tempInt2));
+							else
+								printf("%d bits  ", SSL_get_cipher_bits(ssl, &tempInt2));
 							while (tempInt != 0)
 							{
 								tempInt--;
@@ -584,7 +656,10 @@ int defaultCipher(struct sslCheckOptions *options, SSL_METHOD *sslMethod)
 							}
 							if (options->xmlOutput != 0)
 								fprintf(options->xmlOutput, "%d\" cipher=\"%s\" />\n", SSL_get_cipher_bits(ssl, &tempInt2), SSL_get_cipher_name(ssl));
-							printf("%s\n", SSL_get_cipher_name(ssl));
+							if (options->pout == true)
+								printf("%s ||\n", SSL_get_cipher_name(ssl));
+							else
+								printf("%s\n", SSL_get_cipher_name(ssl));
 
 							// Disconnect SSL over socket
 							SSL_shutdown(ssl);
@@ -991,6 +1066,8 @@ int testHost(struct sslCheckOptions *options)
 	// Test supported ciphers...
 	printf("\n%sTesting SSL server %s on port %d%s\n\n", COL_GREEN, options->host, options->port, RESET);
 	printf("  %sSupported Server Cipher(s):%s\n", COL_BLUE, RESET);
+	if (options->pout == true)
+		printf("|| Status || Version || Bits || Cipher ||\n");
 	sslCipherPointer = options->ciphers;
 	while ((sslCipherPointer != 0) && (status == true))
 	{
@@ -1026,6 +1103,8 @@ int testHost(struct sslCheckOptions *options)
 	{
 		// Test prefered ciphers...
 		printf("\n  %sPrefered Server Cipher(s):%s\n", COL_BLUE, RESET);
+		if (options->pout == true)
+			printf("|| Version || Bits || Cipher ||\n");
 		switch (options->sslVersion)
 		{
 			case ssl_all:
@@ -1081,7 +1160,9 @@ int main(int argc, char *argv[])
 	xmlArg = 0;
 	strcpy(options.host, "127.0.0.1");
 	options.noFailed = false;
+	options.starttls = false;
 	options.sslVersion = ssl_all;
+	options.pout = false;
 	SSL_library_init();
 
 	// Get program parameters
@@ -1110,6 +1191,10 @@ int main(int argc, char *argv[])
 		else if (strncmp("--xml=", argv[argLoop], 6) == 0)
 			xmlArg = argLoop;
 
+		// P Output
+		else if (strcmp("-p", argv[argLoop]) == 0)
+			options.pout = true;
+
 		// Client Certificates
 		else if (strncmp("--certs=", argv[argLoop], 8) == 0)
 			options.clientCertsFile = argv[argLoop] +8;
@@ -1122,6 +1207,13 @@ int main(int argc, char *argv[])
 		else if (strncmp("--pkpass=", argv[argLoop], 9) == 0)
 			options.privateKeyPassword = argv[argLoop] +9;
 
+		// StartTLS...
+		else if (strcmp("--starttls", argv[argLoop]) == 0)
+		{
+			options.sslVersion = tls_v1;
+			options.starttls = true;
+		}
+
 		// SSL v2 only...
 		else if (strcmp("--ssl2", argv[argLoop]) == 0)
 			options.sslVersion = ssl_v2;
@@ -1206,6 +1298,8 @@ int main(int argc, char *argv[])
 			printf("                       PKCS#12 file.\n");
 			printf("  %s--certs=<file>%s       A file containing PEM/ASN1 formatted\n", COL_GREEN, RESET);
 			printf("                       client certificates.\n");
+			printf("  %s--starttls%s           If a STARTTLS is required to kick an\n", COL_GREEN, RESET);
+			printf("                       SMTP service into action.\n");
 			printf("  %s--xml=<file>%s         Output results to an XML file.\n", COL_GREEN, RESET);
 			printf("  %s--version%s            Display the program version.\n", COL_GREEN, RESET);
 			printf("  %s--help%s               Display the  help text  you are  now\n", COL_GREEN, RESET);
